# Matti's Nightmare, v0.1

# Copyright (C) 2019 Hannu Viitala
#
# The source code in this file is released under the MIT license.
# Go to http://opensource.org/licenses/MIT for the full license details.
#

# 13.6.2019, 3 rooms, fps: 64.5, free ram:9776b, rom size:179kb
# 13.6.2019, 3 rooms, mirroring, fps: 60, free ram:9776b, rom size:179kb

import upygame as pygame
import urandom as random
import globals as glob
import data
import rooms
import gameobject as GameObjects
import sprite
import gc
import umachine
from micropython import const
import gc
import utils

# This is needed to be able to change global variables (!)

# Constants

# vieport move rect
HERO_VP_MOVE_TLX  = const(40)
HERO_VP_MOVE_TLY  = const(40)
HERO_VP_MOVE_BRX  = const(110-40)
HERO_VP_MOVE_BRY  = const(80-40)

HERO_SPEED_X  = const(1)

# Setup global variables
glob.viewPortX = 0
glob.viewPortY = 0
glob.tileAttributeArray = bytearray(16)

# Start state
startRoomX = 0
startRoomY = 0
#glob.roomIndex = 5
#heroStartPosX=90
#heroStartPosY=70
#glob.roomIndex = 9
#heroStartPosX=5
#heroStartPosY=70
glob.roomIndex = 7
heroStartPosX=100
heroStartPosY=8

# Create tilemap.
# Note: This has to be at the very beginning, because , for sonme reason, if "print()" was used
# before this it will result garbage when drawing the tilemap
glob.tilemap = pygame.tilemap.Tilemap(glob.TILEMAP_W,glob.TILEMAP_H, data.ramMapPixels)

# Determine if we are running in emulator (or simulator)? 
# The pin Ext2 value is 0 in emulator but not in real HW.
isEmulator = False
joyhatY = umachine.Pin ( umachine.Pin.EXT2, umachine.Pin.ANALOG_IN )
joyhatYValue = joyhatY.value()
#print("joyhatY.value()=",test)
if(joyhatYValue == 0):
    isEmulator = True

screen = pygame.display.set_mode() # full screen
pygame.display.init(False)

# ZX Spectrum palette
pygame.display.set_palette_16bit([
    0, 0x001a, 0xd000, 0xd01a, 0x06a0, 0x06ba, 0xd6a0, 0xd6ba,
    0x0000, 0x001f, 0xf800, 0xf81f, 0x07e0, 0x07ff, 0xffe0, 0xffff
]);


# Initialize sound
glob.sound = pygame.mixer.Sound()
glob.currentBgMusic = "rap2h.raw"
if(isEmulator==False):
    glob.sound.play_from_sd(glob.currentBgMusic)


#########################################
def main():

    # setup sprite group
    glob.all_enemies = sprite.Group()
    glob.all_enemies_list = []

    # Create hero
    glob.heroGob = GameObjects.HeroGameObject(
        [data.herox0, data.herox1, data.herox2, data.herox1 ],
        [[-1,0],[0,0],[-1,0],[-1,0]])
    glob.heroGob.wx = heroStartPosX
    glob.heroGob.wy = heroStartPosY
    #glob.heroGob.wy = 8 + startRoomY
    glob.heroGob.animDur = 1


    # Create enemies
    dummySurf = data.clockSurf0
    for i in range(glob.ENEMIES_MAX_COUNT):
        enemyGob = GameObjects.EnemyBounceGameObject([dummySurf], [[0,0]])
        glob.all_enemies.add(enemyGob)
        glob.all_enemies_list.append(enemyGob)
    
    # Reset room
    utils.ResetRoom(glob.roomIndex)

   # loop variables
    vx = 0
    vy = 0

    # The main loop
    while True:

        glob.currentFrameTime = umachine.time_ms()

        if(glob.isGameLost):
            # Draw the game over window
            pygame.draw.rect(pygame.Rect(1,44-12,108,24), True, 15)
            pygame.draw.rect(pygame.Rect(3,44-10,104,20), True, 8)
            pygame.draw.text(5,44-6,"You did not wake up",15)
            pygame.draw.text(28,44-6+8,"GAME OVER!",15)
            pygame.display.flip()
            continue

        # Read the Pokitto standard keys
        eventtype = pygame.event.poll()
        if eventtype != pygame.NOEVENT:
            if eventtype.type == pygame.KEYDOWN:
                if eventtype.key == pygame.K_RIGHT:
                    vx = HERO_SPEED_X
                if eventtype.key == pygame.K_LEFT:
                    vx = -HERO_SPEED_X
                #if eventtype.key == pygame.K_DOWN:
                #    vy = HERO_SPEED_X
                if eventtype.key == pygame.BUT_A:
                    glob.heroGob.tryJump = True
                    
                if eventtype.key == pygame.BUT_B:
                    glob.sound.reset()
                if eventtype.key == pygame.BUT_C:
                        gc.collect()
                        print ("free C2:",gc.mem_free())

            if eventtype.type == pygame.KEYUP:
                if eventtype.key == pygame.K_RIGHT:
                    vx = 0
                if eventtype.key == pygame.K_LEFT:
                    vx = 0
                #if eventtype.key == pygame.K_UP:
                #    vy = 0
                #if eventtype.key == pygame.K_DOWN:
                #    vy = 0
                #if eventtype.key == pygame.BUT_A:
                #    shoot = False

        # Store velocity
        glob.heroGob.vx = vx
        #glob.heroGob.vy = vy
        
        # Execute expired events
        removeItemCount = 0
        for ev in glob.timedEventList:
            if(ev[0] < glob.currentFrameTime):
                print("execute timed event", ev)
                removeItemCount += 1
                if(ev[1] == glob.TIMED_EVENT_STOP_MUSIC):
                    print("execute timed event. Reset music")
                    glob.sound.reset()
                    if(isEmulator==False):
                        glob.sound.play_from_sd("dummy123.raw")
            else:
                break
            
        # Remove all expired event
        for i in range(removeItemCount):
            glob.timedEventList.pop(0)
                   

        # *** UPDATE ***
        
        # rare update
        #if(glob.frameNum%20==0):
        #RoomUpdate()

        # Update hero
        #glob.heroGob.update()
        glob.heroGob.update()
        #heroGob3.update()

        # Move viewport
        MoveViewport()

        # Update enemies
        glob.all_enemies.update()

        # *** DRAW ***

        # Draw tilemap
        glob.tilemap.draw(glob.viewPortX, glob.viewPortY)

        # Draw enemies
        glob.all_enemies.draw(screen)

        # Draw hero
        if(glob.heroGob.visible):
            screen.blit(glob.heroGob.image, glob.heroGob.rect.x, glob.heroGob.rect.y, 0, glob.heroGob.hmirror)

        #
        DrawStatusInfo()

        pygame.display.flip()

        glob.frameNum += 1

        #umachine.wait(100)

#########################################
def DrawStatusInfo():

    # Draw gold bitmap and count
    screen.blit( data.goldSurf, 0, 0)
    screen.blit( data.numberBackgroundSurf, 7,0)
    pygame.draw.text(8,1,str(glob.goldCount),15); # white number

    # Draw heart bitmap and count
    screen.blit( data.heartSurf, 110-16, 0)
    screen.blit( data.numberBackgroundSurf, 110-8,0)
    pygame.draw.text(110-7,1,str(glob.hartCount),15); # white number

    # Draw room name
    pygame.draw.rect(pygame.Rect(0,81,110,7), True, 8);
    pygame.draw.text(10,83,glob.roomParameters[glob.roomIndex][0],15);
    
    
    # Collect all freed memory. 
    # gc.collect()
    # pygame.draw.text(10,10,str(gc.mem_free()),15);


#########################################
def MoveViewport():

    # Calculate viewport position
    
    # If the hero is outside the viewport, move the viewport
    hero_vpx = glob.heroGob.rect.x
    dx = HERO_VP_MOVE_TLX - hero_vpx
    vpx = 0
    if(dx < 0):
        # not moved too much left
        vpx = 0  # do not move vp
        dx = HERO_VP_MOVE_BRX - hero_vpx
        if(dx > 0):
            # not moved too much right
            vpx = 0  # do not move viewport
        else:
            vpx = dx # move viewport
    else:
        vpx = dx # move viewport
    hero_vpy = glob.heroGob.rect.y
    dy = HERO_VP_MOVE_TLY - hero_vpy
    vpy = 0
    if(dy < 0):
        # not moved too much up
        vpy = 0
        dy = HERO_VP_MOVE_BRY - hero_vpy
        if(dy > 0):
            # not moved too much down
            vpy = 0  # do not move viewport
        else:
            vpy = dy # move viewport
    else:
        vpy = dy # move viewport

    # Move viewport
    newVpX = glob.viewPortX + vpx
    newVpY = glob.viewPortY + vpy

    # Check that viewport is not out bounds
    if(newVpX>0):
        newVpX=0
    elif(newVpX + glob.TILEMAP_W_PX < glob.SCREEN_W):
        newVpX = glob.SCREEN_W - glob.TILEMAP_W_PX
    if(newVpY>0):
        newVpY=0
    elif(newVpY + glob.TILEMAP_H_PX < glob.SCREEN_H-7):
        newVpY = glob.SCREEN_H-7 - glob.TILEMAP_H_PX
    glob.viewPortX = newVpX
    glob.viewPortY = newVpY

    # update hero's position in the viewport
    glob.heroGob.rect.x = glob.heroGob.wx + glob.viewPortX
    glob.heroGob.rect.y = glob.heroGob.wy + glob.viewPortY

#####################################

# Start mai
#premain()
main()
